-- stage
stage:setClearColorBuffer(bool)
-- some vars
local myappwidth, myappheight = application:getContentWidth(), application:getContentHeight()
-- collisions BITS
local PLAYER1_BIT = math.pow(2,0)
local ENEMY_BIT = math.pow(2,1)
local MISSILE_BIT = math.pow(2,2)
-- keyboard control
local isup, isdown, isleft, isright, isshoot = false, false, false, false, false
-- the score
local score, lives = 0, 3
local scoretf = TextField.new(nil, "SCORE: "..score)
scoretf:setScale(3)
scoretf:setTextColor(0x00ffff)
scoretf:setPosition(myappwidth - scoretf:getWidth() - 32, scoretf:getHeight() + 8)
local livestf = TextField.new(nil, "LIVES: "..lives)
livestf:setScale(3)
livestf:setTextColor(0x00ffff)
livestf:setPosition(32, livestf:getHeight() + 8)
-- r3d
r3d = require "reactphysics3d"
local gravityy = 0
local world = r3d.World.new(0, gravityy, 0) -- 0, -9.8, 0
world.missiles = {} -- missiles list
world.enemies = {} -- enemies list
-- a fullscreen 3D viewport
local camera = D3.View.new(myappwidth, myappheight, 50, 1, 512) -- function D3View:init(width, height, fov, near, far)
--[[
-- a sky
local sky = Pixel.new(0xffffff, 1, myappwidth * 2, myappheight * 2)
sky:setAnchorPoint(0.5, 0.5)
sky:setPosition(myappwidth / 2, myappheight / 2)
sky:setColor(0xfffff0, 1, 0xff44ff, 1, 90) -- gradient background
stage:addChild(sky)
]]
-- the background
local bgtex = Texture.new("3d_models/texs/bg_fall02.png", true, {wrap=Texture.REPEAT})
local bg = Pixel.new(bgtex, nil, nil, 1, 1)
bg:setDimensions(myappwidth, myappheight)
-- the player
local player1x = PLAYER1.new(world, "3d_models/mkship", "mkship.obj",
		{posz=-14, mass=32}, PLAYER1_BIT, ENEMY_BIT)
local player1, player1body = player1x:get()
-- add the objects to the scene
stage:addChild(bg)
camera:getScene():addChild(player1)
camera:lookAt(-48,0,0, player1:getPosition())
stage:addChild(camera)
stage:addChild(scoretf)
stage:addChild(livestf)

-- collisions listener
world:setEventListener(function()
	for k, v in pairs(world.missiles) do
		for k1, v1 in pairs(world.enemies) do
			if world:testAABBOverlap(v.body, v1.body) then
				v.isdirty, v1.isdirty = true, true
				score += 10
				scoretf:setText("SCORE: "..score)
			end
		end
	end
	for k, v in pairs(world.enemies) do
		if world:testAABBOverlap(v.body, player1body) then
			v.isdirty = true
			lives -= 1
			livestf:setText("LIVES: "..lives)
		end
	end
end)

-- the game loop
local speed, missilespeed = 20 * 32, 1 * 32
local missilelimit, maxmissilelimit = 0, 4 * 8
local enemylimit, maxenemylimit = 0, 7 * 8
local spawntimer = 256
local bgadv = 0
stage:addEventListener(Event.ENTER_FRAME, function(e)
	-- the bg
	bgadv += 0.25
	bg:setTexturePosition(bgadv,0)
	-- destroy
	for k, v in pairs(world.missiles) do
		missilelimit = -(player1:getZ() - k:getZ()) <> (player1:getZ() - k:getZ())
		if missilelimit > maxmissilelimit or v.isdirty then
			world.missiles[k] = nil
			world:destroyBody(v.body)
			camera:getScene():removeChild(k)
		end
	end
	for k, v in pairs(world.enemies) do
		enemylimit = -(player1:getZ() - k:getZ()) <> (player1:getZ() - k:getZ())
		if enemylimit > maxenemylimit or v.isdirty then
			world.enemies[k] = nil
			world:destroyBody(v.body)
			camera:getScene():removeChild(k)
		end
	end
	-- action!
	world:step(e.deltaTime)
	-- the player
	if isup then player1body:applyForce(0, speed, 0) end
	if isdown then player1body:applyForce(0, -speed, 0) end
	if isleft then player1body:applyForce(0, 0, speed) end
	if isright then player1body:applyForce(0, 0, -speed) end
	if isshoot then shoot() isshoot = false end
	player1:setMatrix(player1body:getTransform())
	-- the missiles
	for k, v in pairs(world.missiles) do
		v.body:applyForce(0, 0, missilespeed)
		v.body:applyTorque(0, 0, 2)
		k:setMatrix(v.body:getTransform())
	end
	-- the enemies
	for k, v in pairs(world.enemies) do
		v.body:applyForce(0,0,-speed * 0.75)
		k:setMatrix(v.body:getTransform())
	end
	spawntimer -= 1
	if spawntimer <= 0 then spawnEnemy() spawntimer = 300 end
end)

-- spawner
function spawnEnemy()
	local enemyx = ENEMY.new( world, "3d_models/enemies", "ship03.obj",
		{ posx=0, posy=Core.random(0,-4,4), posz=Core.random(0,18,20), roty=180, mass=32 },
		ENEMY_BIT, MISSILE_BIT+PLAYER1_BIT )
	local enemy, enemybody = enemyx:get()
	enemy:setMatrix(enemybody:getTransform())
	camera:getScene():addChild(enemy)
end

-- shoot
function shoot()
	local missilex = MISSILE.new( world, "3d_models/missiles", "missile03.obj",
		{ posx=player1:getX(), posy=player1:getY(), posz=player1:getZ() + 2, mass=0.25 },
		MISSILE_BIT, ENEMY_BIT )
	local missile = missilex:get()
	camera:getScene():addChild(missile)
end

-- keyboard
stage:addEventListener(Event.KEY_DOWN, function (e)
	if e.keyCode == KeyCode.UP then isup = true end
	if e.keyCode == KeyCode.DOWN then isdown = true end
	if e.keyCode == KeyCode.LEFT then isleft = true end
	if e.keyCode == KeyCode.RIGHT then isright = true end
	if e.keyCode == KeyCode.SPACE then isshoot = true end
end)

stage:addEventListener(Event.KEY_UP, function (e)
	if e.keyCode == KeyCode.UP then isup = false end
	if e.keyCode == KeyCode.DOWN then isdown = false end
	if e.keyCode == KeyCode.LEFT then isleft = false end
	if e.keyCode == KeyCode.RIGHT then isright = false end
	if e.keyCode == KeyCode.SPACE then isshoot = false end
end)
